﻿  // @file util.cc
  // @brief Useful common functions
  // @author Mamoru Kaminaga
  // @date 2017-12-15 11:38:55
  // Copyright 2017 Mamoru Kaminaga
  // Released under the MIT license
  // http://opensource.org/licenses/mit-license.php
#include <stdint.h>
#include <stdio.h>
#include <time.h>
#include <wchar.h>
#include <string>
#include "./util.h"

std::basic_string<char> GetMBS(const wchar_t* format, ...) {
  char mbs_buf[UTIL_MAXSTRLEN] = {0};
  wchar_t wcs_buf[UTIL_MAXSTRLEN] = {0};
  va_list args;
  va_start(args, format);

  // Variable arguments are processed.
#if defined(WIN32)
  size_t value = 0;
  vswprintf_s(wcs_buf, UTIL_MAXSTRLEN, format, args);
  wcstombs_s(&value, mbs_buf, UTIL_MAXSTRLEN, wcs_buf, UTIL_MAXSTRLEN);
#elif defined(UNIX)
  vswprintf(wcs_buf, UTIL_MAXSTRLEN, format, args);
  wcstombs(mbs_buf, wcs_buf, UTIL_MAXSTRLEN);
#endif

  return mbs_buf;
}
std::basic_string<wchar_t> GetWCS(const char* format, ...) {
  char mbs_buf[UTIL_MAXSTRLEN] = {0};
  wchar_t wcs_buf[UTIL_MAXSTRLEN] = {0};
  va_list args;
  va_start(args, format);

  // Variable arguments are processed.
#if defined(WIN32)
  size_t value = 0;
  vsprintf_s(mbs_buf, UTIL_MAXSTRLEN, format, args);
  mbstowcs_s(&value, wcs_buf, UTIL_MAXSTRLEN, mbs_buf, UTIL_MAXSTRLEN);
#elif defined(UNIX)
  vsnprintf(mbs_buf, UTIL_MAXSTRLEN, format, args);
  mbstowcs(wcs_buf, mbs_buf, UTIL_MAXSTRLEN);
#endif

  return wcs_buf;
}
FILE* Fopen(const wchar_t* file_name, const wchar_t* mode) {
  FILE* fp = nullptr;
#if defined(WIN32)
  _wfopen_s(&fp, file_name, mode);
#elif defined(UNIX)
  fp = fopen(GetMBS(file_name).c_str(), GetMBS(mode).c_str());
#endif
  return fp;
}
int ReadByte(FILE* fp, RWMODE mode) {
  int value = 0;
  switch (mode) {
    case RWMODE_DEC:
#if defined(WIN32)
      fwscanf_s(fp, L"%d", &value);
#elif defined(UNIX)
      fwscanf(fp, L"%d", &value);
#endif
      break;
    case RWMODE_HEX:
      {
        wchar_t a[3] = {0};
        a[0] = fgetwc(fp);
        a[1] = fgetwc(fp);
        a[2] = L'\0';
        wchar_t* end_ptr = nullptr;
        value = wcstol(a, &end_ptr, 16);
      }
      break;
    default:
      // No implementation.
      break;
  }
  return value;
}
void WriteByte(FILE* fp, int value, RWMODE mode) {
  int ai = 0;  // The upper 4 bits data.
  int bi = 0;  // The lower 4 bits data.
  int a = 0;  // The upper 4bits ascii index.
  int b = 0;  // The lower 4bits ascii index.
  switch (mode) {
    case RWMODE_DEC:
      fwprintf(fp, L"%d ", value);
      break;
    case RWMODE_HEX:
      fwprintf(fp, L"%02X", value);
      break;
    case RWMODE_CMD:
      ai = value >> 4;
      if ((ai >= 0) || (ai <= 9)) {
        a = static_cast<int>(L'0') + ai;
      } else {
        a = static_cast<int>(L'A') + ai - 10;
      }
      bi = value & 0x0f;
      if ((bi >= 0) || (bi <= 9)) {
        b = static_cast<int>(L'0') + bi;
      } else {
        b = static_cast<int>(L'A') + bi - 10;
      }
      fwprintf(fp, L"\"%d\",\"%d\",", a, b);
      break;
    default:
      // No implementation.
      break;
  }
}
void GetLocalTime(struct tm* local_time) {
  time_t timer;
  time(&timer);
#if defined(WIN32)
  localtime_s(local_time, &timer);
#elif defined(UNIX)
  localtime_r(&timer, local_time);
#endif
}
